searchData={"items":[{"type":"module","title":"Decibel","doc":"`Decibel` is an implementation of [The Noise Protocol Framework](https://noiseprotocol.org).\n\n> Noise is a framework for building crypto protocols. Noise protocols support\n> mutual and optional authentication, identity hiding, forward secrecy, zero\n> round-trip encryption, and other advanced features.\n\nFor more information about Noise, its rationale, supported protocols etc,\nplease refer to [The Noise Specification](https://noiseprotocol.org/noise.html).\n\nThe rest of this document assumes the reader is familiar with the above\nspecification.","ref":"Decibel.html"},{"type":"module","title":"Overview - Decibel","doc":"Decibel encrypts and decrypts messages according to the Noise Protocol,\nand the client's selection of handshake and cryptographic primitives. It does\n_not_ act as a transport, nor does it say anything about how Noise messages\nshould be transmitted between participants.\n\nEach party - either the _initiator_ (the party that starts the handshake) or\n_responder_ (the other party) - advances the handshake until it completes, at\nwhich point a secure, symmetric channel is established that either party may\nuse to encrypt and decrypt outbound and inbound messages respectively.\n\nDecibel supports all the handshake patterns outlined in r34 of the specification\nincluding the fundamental patterns, deferred patterns and one-way patterns. It\nalso supports pre-shared keys as outlined in the specification, and fallback\nhandshakes for [Noise Pipes](http://www.noiseprotocol.org/noise.html#noise-pipes)\nsupport.","ref":"Decibel.html#module-overview"},{"type":"module","title":"Example - Decibel","doc":"Consider the following handshake defined in the Noise Protocol:\n```text\nNN:\n  -> e\n  <- e, ee\n```\n\nThe parties agree on this handshake and its cryptographic parameters and\nexpress this in a _protocol name_, e.g. `Noise_NN_25519_AESGCM_SHA256`. The\n_initiator's_ code may look something like this:\n\n```elixir\n# Create the protocol instance\nini = Decibel.new(\"Noise_NN_25519_AESGCM_SHA256\", :ini)\n# Perform the first stage of the handshake\nmsg1 = Decibel.handshake_encrypt(ini)\n# Somehow send this message to the responder and get the response\nmagically_send_msg(rsp_proc, msg1)\nmsg2 = magically_recv_msg(rsp_proc)\n# Process the response through the second stage\nDecibel.handshake_decrypt(ini, msg2)\n# At this point, the 'NN' handshake has completed for the initiator\n# and regular messages may be sent and received\nmsg3 = Decibel.encrypt(ini, \"Hello, world\")\nmagically_send_msg(rsp_proc, msg3)\n```\nThe _responder's_ code may look something like this:\n\n```elixir\n# Create the protocol instance\nrsp = Decibel.new(\"Noise_NN_25519_AESGCM_SHA256\", :rsp)\n# Receive the first-stage message from the initiator\nmsg1 = magically_recv_msg(ini_proc)\n# Process the message through the protocol\nDecibel.handshake_decrypt(rsp, msg1)\n# Send the second stage to the initiator\nmsg2 = Decibel.handshake_encrypt(rsp)\nmagically_send_msg(ini_proc, msg2)\n# At this point, the 'NN' handshake has completed for the responder\n# and regular messages may be sent and received\nmsg3 = magically_recv_msg(ini_proc)\n\"Hello, world\" = Decibel.decrypt(rsp, msg3)\n```","ref":"Decibel.html#module-example"},{"type":"module","title":"Lifecycle - Decibel","doc":"","ref":"Decibel.html#module-lifecycle"},{"type":"module","title":"Creation - Decibel","doc":"Each party begins by creating a new handshake, via `new/4`, specifying the\n[protocol name](https://noiseprotocol.org/noise.html#protocol-names-and-modifiers),\nthe role the party plays in the handshake (`:ini` for initiator, `:rsp` for\nresponder), and optionally any pre-message keys.\n\n```\n# In the IK handshake, the responder's public (static) key is known to the\n# initiator prior to the handshake.\nkeys = %{rs: <<...>>}\nini  = Decibel.new(\"Noise_IK_448_ChaChaPoly_BLAKE2b\", :ini, keys)\n```\n\nThe result of `new/4` is a reference used for the rest of the session.","ref":"Decibel.html#module-creation"},{"type":"module","title":"Handshake - Decibel","doc":"During the handshake phase, the protocol is advanced by each party in turn. For\ninitiators, this typically starts with calling `handshake_encrypt/2` and sending\nthe result to the responder. In turn, the responder calls `handshake_decrypt/2`\nbefore typically encrypting its own handshake message and sending that to the\ninitiator.\n\nThis sequence continues until the handshake is complete. If the selected protocol\nis known at compile time, the parties can just assume its completion in the\nabsence of an error (as in the example [above](#module-example)). Alternatively,\neach party can call `is_handshake_complete?/1` after each handshake\nencryption/decryption.\n\nOnce the handshake is complete, a secure channel is established with the\n[properties](https://noiseprotocol.org/noise.html#payload-security-properties) of\nthe selected protocol.\n\nAdditionally, once the handshake is complete, a unique 'session-hash' is available\nvia `get_handshake_hash/1` - see the [channel-binding](https://noiseprotocol.org/noise.html#channel-binding)\nsection of the specification for more details.","ref":"Decibel.html#module-handshake"},{"type":"module","title":"Session - Decibel","doc":"Once the handshake is complete, the parties use `encrypt/3` and `decrypt/3` to\nexchange 'application' messages between each other. Both functions provide for optional\n'associated authenticated data' to be specified, that provides message-integrity\nassurance for the application data.\n\nOnce the session is complete, each party should call `close/1` to free the\nresources associated with the it.","ref":"Decibel.html#module-session"},{"type":"module","title":"Noise Pipes - Decibel","doc":"[Noise Pipes](http://www.noiseprotocol.org/noise.html#noise-pipes) are compound protocols\ncombining:\n\n- A full handshake (e.g. `XX`)\n- A zero-RTT handshake (e.g. `IK`)\n- A fallback handshake (e.g. `XXfallback`)\n\nThe specification provides more detail on Noise Pipes, as does the\n[Wiki](https://github.com/noiseprotocol/noise_wiki/wiki/Test-vectors#noise-pipes).\nDecibel provides support for all these individual protocols and the necessary\ninformation to transition between a failed IK handshake and the fallback.","ref":"Decibel.html#module-noise-pipes"},{"type":"module","title":"Decryption Errors - Decibel","doc":"If an AEAD decryption failure occurs, a `Decibel.DecryptionError` is raised.\nAdditionally if this error occurs during a handshake, the error's `:remote_keys`\nproperty will contain any remote public keys processed during the handshake, up\nto the point of failure.","ref":"Decibel.html#module-decryption-errors"},{"type":"module","title":"Example - Decibel","doc":"The following example shows a responder handling the decryption failure, and then\ntransitioning to the fallback protocol, using the remote ephemeral key, via\n`Noise_XXfallback_25519_ChaChaPoly_Blake2b`.\n\n```elixir\n# Process IK handshake message sent by the initiator\ntry do\n  _ = Decibel.handshake_decrypt(rsp, ciphertext)\n  # Happy path continues here...\nrescue\n  e in Decibel.DecryptionError ->\n    # Grab the remote ephemeral key sent by the initiator during the failed\n    # handshake\n    re = e.remote_keys[:re]\n    # Now construct the new responder for the fallback protocol\n    rsp = Decibel.new(\"Noise_XXfallback_25519_ChaChaPoly_Blake2b\", :rsp, %{re: re}, swap: :rsp)\n    # Start the new handshake (not shown) ...\nend\n```\n\nNote that although the code reconstructs the responder, as the handshake is a fallback\nprotocol, the code is _effectively_ the initiator, and will send the first message on\nthis new handshake.\n\n- The retrievel of the remote ephemeral (`re`) key from the error\n- The prepopulation of that key in the responder's new handshake (other keys omitted\n  for brevity)\n- The use of the `[swap: :rsp]` option - this is required to ensure the split cipher\n  channels are correctly paired after the handshake.","ref":"Decibel.html#module-example"},{"type":"module","title":"Connectionless Transports - Decibel","doc":"Once the handshake completes, Noise provides support for the encryption and decryption\nof messages over connectionless i.e. potentially _unordered_, potentially _lossy_\ntransports, and Decibel honours this support.\n\nThis example shows how to send data over such a transport:\n\n```elixir\n# First, grab the nonce for the outbound channel\nn = Decibel.get_nonce(ref, :out)\n# Encrypt the data\nciphertext = Decibel.encrypt(ref, plaintext, aad)\n# Send both the nonce and the ciphertext\nsend(peer, {n, ciphertext})\n```\n\nThe receiving side is as follows:\n\n```elixir\n# Receive the message\n{n, ciphertext} = get_msg_from(peer)\n# Set the nonce for the inbound channel using the received n\n:ok = Decibel.set_nonce(ref, :in, n)\n# Decrypt the ciphertext\nplaintext = Decibel.decrypt(ref, ciphertext, aad)\n```","ref":"Decibel.html#module-connectionless-transports"},{"type":"function","title":"Decibel.close/1","doc":"Release the resources associated with the session.\n\nThese resources are automatically released when the process terminates, but\nthis call may be used to eagerly clean them up.","ref":"Decibel.html#close/1"},{"type":"function","title":"Decibel.decrypt/3","doc":"Decrypts a message over an established session, using an optionally\nprovided AAD for message integrity.\n\nReturns the decrypted message, or raises a `RuntimeException` if the\nmessage cannot be decrypted.","ref":"Decibel.html#decrypt/3"},{"type":"function","title":"Decibel.encrypt/3","doc":"Encrypts a message over an established session, using an optionally\nprovided AAD for message integrity.\n\nReturns the encrypted message.","ref":"Decibel.html#encrypt/3"},{"type":"function","title":"Decibel.get_handshake_hash/1","doc":"Returns a 32-byte handshake hash, unique to the established session.\n\nReturns `nil` if the handshake is not yet completed.","ref":"Decibel.html#get_handshake_hash/1"},{"type":"function","title":"Decibel.get_nonce/2","doc":"Get the current nonce value of the specified cipher.","ref":"Decibel.html#get_nonce/2"},{"type":"function","title":"Decibel.get_remote_key/1","doc":"Get the remote (static) key if available.","ref":"Decibel.html#get_remote_key/1"},{"type":"function","title":"Decibel.handshake_decrypt/2","doc":"Decrypt an inbound handshake message, returning any optionally provided application\ndata.\n\nThe function will raise a `Decibel.DecryptionError` if the handshake data does not\ndecrypt correctly.","ref":"Decibel.html#handshake_decrypt/2"},{"type":"function","title":"Decibel.handshake_encrypt/2","doc":"Encrypt an outbound handshake message, optionally folding in application data.\n\n> The reader is encouraged to understand the ramifications of providing application\n> data _during_ the handshake. As the handshake is not yet completed, the properties\n> of any secure channel have not yet been established. Such data may even be sent in\n> the clear. Consult the [Payload Security Properties](https://noiseprotocol.org/noise.html#payload-security-properties)\n> in the specification for more information.","ref":"Decibel.html#handshake_encrypt/2"},{"type":"function","title":"Decibel.is_handshake_complete?/1","doc":"Returns `true` if the handshake is complete, `false` otherwise.","ref":"Decibel.html#is_handshake_complete?/1"},{"type":"function","title":"Decibel.new/4","doc":"Start a new handshake.\n\nThe caller should provide a [protocol name](https://noiseprotocol.org/noise.html#protocol-names-and-modifiers)\nand the role the caller will play in the protocol. The caller should provide any keys\nrequired by the protocol prior to advancing the handshake. This are typically either\nstatic keys or pre-shared keys (PSKs), but ephemeral keys may also be provided. The\nlist of provided keys should be identified as follows:\n\n- `:s`: the party's public-private static key pair as a tuple.\n- `:rs`: the peer's public static key as a binary.\n- `:psks`: a list of [pre-shared symmetric keys](https://noiseprotocol.org/noise.html#pre-shared-symmetric-keys)\n(as binaries), one for each psk modifier.\n- `:prologue`: any [prologue](https://noiseprotocol.org/noise.html#prologue) data\n\nThis function will raise an exception if any required keys are missing.\n\nReturns a reference representing the handshake.","ref":"Decibel.html#new/4"},{"type":"function","title":"Decibel.rekey/2","doc":"Rekey the inbound or outbound channel of the session.","ref":"Decibel.html#rekey/2"},{"type":"type","title":"Decibel.role/0","doc":"The role the party plays in the protocol.","ref":"Decibel.html#t:role/0"},{"type":"function","title":"Decibel.set_nonce/3","doc":"Set the current value of nonce for the specified cipher.","ref":"Decibel.html#set_nonce/3"},{"type":"exception","title":"Decibel.DecryptionError","doc":"Represents a decryption failure.\n\nIf the failure occurs during the handshake phase, the `:remote_keys`\nfield will contain any remote public keys used in the handshake.","ref":"Decibel.DecryptionError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.2.4 - Jan 22, 2025 - Changelog","doc":"- Update deps\n- Fix compiler errors","ref":"changelog.html#0-2-4-jan-22-2025"},{"type":"extras","title":"0.2.3 - June 6, 2023 - Changelog","doc":"- Fix unused alias warning.","ref":"changelog.html#0-2-3-june-6-2023"},{"type":"extras","title":"0.2.2 - June 6, 2023 - Changelog","doc":"- Add `get_remote_key/1` to the public API.","ref":"changelog.html#0-2-2-june-6-2023"},{"type":"extras","title":"0.2.1 - June 6, 2023 (Reverted) - Changelog","doc":"- Add `get_public_key/2` to the public API.","ref":"changelog.html#0-2-1-june-6-2023-reverted"},{"type":"extras","title":"0.2.0 - May 19, 2023 - Changelog","doc":"- Breaking change - renamed `set_n/3` and `get_n/2` to `set_nonce/3`\n  and `get_nonce/2` respectively\n- AEAD failure now raises a `Decibel.DecryptionError` rather than\n  a `RuntimeError`. If this is raised during a handshake, this struct \n  will also contain any remote public keys processed during the\n  handshake, up to the point of failure\n- Added fallback tests\n- Improved documentation around Noise Pipes and connectionless\n  transports","ref":"changelog.html#0-2-0-may-19-2023"},{"type":"extras","title":"0.1.1 - April 25, 2023 - Changelog","doc":"- Initial revision","ref":"changelog.html#0-1-1-april-25-2023"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}